package com.tzld.piaoquan.recommend.feature.service;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.tzld.piaoquan.recommend.feature.model.common.Result;
import com.tzld.piaoquan.recommend.feature.model.feature.FeatureKeyProto;
import com.tzld.piaoquan.recommend.feature.model.feature.MultiGetFeatureRequest;
import com.tzld.piaoquan.recommend.feature.model.feature.MultiGetFeatureResponse;
import com.tzld.piaoquan.recommend.feature.util.CommonCollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author dyp
 */
@Service
@Slf4j
public class FeatureV2Service {
    @Qualifier("redisTemplate")
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @ApolloJsonValue("${dts.config:}")
    private Map<String, List<String>> tableFieldConfig;

    public MultiGetFeatureResponse multiGetFeature(MultiGetFeatureRequest request) {
        if (request.getFeatureKeyCount() == 0) {
            return MultiGetFeatureResponse.newBuilder()
                    .setResult(Result.newBuilder().setCode(1))
                    .build();
        }
        // 目前都在一个Redis，所以放在一个list简化处理
        List<String> redisKeys = CommonCollectionUtils.toList(request.getFeatureKeyList(), fk -> redisKey(fk));
        List<String> values = redisTemplate.opsForValue().multiGet(redisKeys);

        Map<String, String> feature = new HashMap<>();
        for (int i = 0; i < request.getFeatureKeyCount(); i++) {
            feature.put(request.getFeatureKeyList().get(i).getUniqueKey(), values.get(i));
        }

        MultiGetFeatureResponse.Builder builder = MultiGetFeatureResponse.newBuilder();
        builder.setResult(Result.newBuilder().setCode(1));
        builder.putAllFeature(feature);
        return builder.build();
    }

    // Note：写入和读取的key生成规则应保持一致
    private String redisKey(FeatureKeyProto fk) {
        List<String> fields = tableFieldConfig.get(fk.getTableName());
        StringBuilder sb = new StringBuilder(fk.getTableName());
        for (String field : fields) {
            sb.append(":");
            sb.append(fk.getFieldValueMap().get(field));
        }
        return sb.toString();
    }

}
